package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.os.Build
import android.util.Log
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.material3.Button
import androidx.compose.material3.Checkbox
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.session.TaskPermission
import com.faraphel.tasks_valider.database.entities.*
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import java.time.Instant

/**
 * This screen represent a student
 * @param student the student object
 */
@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun TaskStudentScreen(
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
    student: PersonEntity,
) {
    val tasks = remember { mutableStateOf<List<TaskEntity>?>(null) }
    val validations = remember { mutableStateOf<List<ValidationEntity>?>(null) }

    if (tasks.value == null || validations.value == null)
        Thread {
            refreshTasksValidations(
                activity,
                client,
                student,
                tasks,
                validations
            )
        }.start()

    Column {
        Text(text = student.fullName())

        // if both the list of tasks and validations are loaded
        if (!(tasks.value == null || validations.value == null)) {
            tasks.value?.forEach { task ->
                // get the validation
                val validation = validations.value!!.firstOrNull { validation -> validation.taskId == task.id }

                Button(onClick = {}) {
                    Row {
                        Column {
                            // task title
                            Text(task.title)
                            // task description
                            task.description?.let { description -> Text(description) }
                            // if the task have been validated, show the date
                            if (validation != null) Text(validation.date.toString())
                        }
                        // the validation state
                        Checkbox(
                            checked = validation != null,
                            enabled = user.role.permissions.contains(TaskPermission.WRITE),
                            onCheckedChange = { state ->
                                Thread {
                                    updateValidation(
                                        client,
                                        state,
                                        validation ?: ValidationEntity(
                                            Instant.now(),
                                            user.id,
                                            student.id,
                                            task.id,
                                        )
                                    )
                                }.start()
                            }
                        )
                    }
                }
            }
        }
    }
}

fun refreshTasksValidations(
    activity: Activity,
    client: TaskClient,
    student: PersonEntity,
    tasks: MutableState<List<TaskEntity>?>,
    validations: MutableState<List<ValidationEntity>?>,
) {
    val jsonParser = Gson()

    // try to obtain the list of subject
    val responseSubjects = client.get("entities/" + RelationPersonSessionSubjectEntity.TABLE_NAME)

    // in case of error, notify it
    if (!responseSubjects.isSuccessful)
        return activity.runOnUiThread { Toast.makeText(activity, responseSubjects.message, Toast.LENGTH_LONG).show() }

    // parse the list of subjects
    val allPersonSessionSubjects = jsonParser.fromJson<List<RelationPersonSessionSubjectEntity>>(
        responseSubjects.body.string(),
        object : TypeToken<List<RelationPersonSessionSubjectEntity>>(){}.type
    )

    // get the subject that the student is using
    val relationPersonSessionSubjects = allPersonSessionSubjects.firstOrNull { relation -> relation.studentId == student.id }

    if (relationPersonSessionSubjects == null)
        // TODO(Faraphel): should be able to assign a subject ?
        return activity.runOnUiThread { Toast.makeText(activity, "No subject assigned", Toast.LENGTH_LONG).show() }

    // try to obtain the list of tasks
    val responseTasks = client.get("entities/" + TaskEntity.TABLE_NAME)

    // in case of error, notify it
    if (!responseTasks.isSuccessful)
        return activity.runOnUiThread { Toast.makeText(activity, responseTasks.message, Toast.LENGTH_LONG).show() }

    // parse the list of subjects
    val allTasks = jsonParser.fromJson<List<TaskEntity>>(
        responseTasks.body.string(),
        object : TypeToken<List<TaskEntity>>(){}.type
    )

    // get the tasks that are linked to this subject
    tasks.value = allTasks.filter { task ->
        task.subjectId == relationPersonSessionSubjects.subjectId
    }.sortedBy { task ->
        task.order
    }

    // try to obtain the list of validations
    val responseValidations = client.get("entities/" + ValidationEntity.TABLE_NAME)

    // in case of error, notify it
    if (!responseValidations.isSuccessful)
        return activity.runOnUiThread { Toast.makeText(activity, responseTasks.message, Toast.LENGTH_LONG).show() }

    // parse the list of validations
    val allValidations = jsonParser.fromJson<List<ValidationEntity>>(
        responseValidations.body.string(),
        object : TypeToken<List<ValidationEntity>>(){}.type
    )

    // filter only the interesting validations
    validations.value = allValidations.filter { validation ->
        validation.studentId == student.id &&
        validation.taskId in allTasks.map { task -> task.id }
    }
}


fun updateValidation(client: TaskClient, checked: Boolean, validation: ValidationEntity) {
    val jsonParser = Gson()

    if (checked) {
        // if the validation is not set, create it
        client.post(
            "entities/" + ValidationEntity.TABLE_NAME,
            jsonParser.toJson(validation),
            "application/json"
        )
    }
    else {
        // if the validation is set, delete it
        client.delete(
            "entities/" + ValidationEntity.TABLE_NAME,
            jsonParser.toJson(validation),
            "application/json"
        )
    }
}